Package: tinyarray
Type: Package
Title: Expression Data Analysis and Visualization
Version: 2.4.3
Authors@R: person("Xiaojie", "Sun", email = "18763899370@163.com", role = c("aut", "cre"))
Maintainer: Xiaojie Sun <18763899370@163.com>
Description: The Gene Expression Omnibus (<https://www.ncbi.nlm.nih.gov/geo/>) and The Cancer Genome Atlas (<https://portal.gdc.cancer.gov/>) are widely used medical public databases. Our platform integrates routine analysis and visualization tools for expression data to provide concise and intuitive data analysis and presentation.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: BiocManager, clusterProfiler, dplyr, limma, stringr, tibble,
        pheatmap, ggplot2, survival, Hmisc, survminer, patchwork
Suggests: testthat, AnnoProbe, GEOquery, Biobase, VennDiagram,
        FactoMineR, factoextra, knitr, rmarkdown, cowplot, ggpubr,
        ggplotify, tidyr, labeling, Rtsne, scatterplot3d,
        ComplexHeatmap, circlize,org.Rn.eg.db,org.Mm.eg.db,org.Hs.eg.db
URL: https://github.com/xjsun1221/tinyarray
BugReports: https://github.com/xjsun1221/tinyarray/issues
Depends: R (>= 4.1.0)
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-03-05 09:01:31 UTC; xiaojie
Author: Xiaojie Sun [aut, cre]
Repository: CRAN
Date/Publication: 2025-03-05 13:20:02 UTC
Built: R 4.6.0; ; 2026-05-01 06:16:02 UTC; unix
