# Generated by roxygen2: do not edit by hand

export(box_surv)
export(cor.full)
export(cor.one)
export(corheatmap)
export(corscatterplot)
export(double_enrich)
export(draw_KM)
export(draw_boxplot)
export(draw_heatmap)
export(draw_heatmap2)
export(draw_pca)
export(draw_tsne)
export(draw_venn)
export(draw_volcano)
export(draw_volcano2)
export(dumd)
export(edges_to_nodes)
export(exp_boxplot)
export(exp_surv)
export(find_anno)
export(geo_download)
export(geo_parser)
export(get_cgs)
export(get_count_txt)
export(get_deg)
export(get_deg_all)
export(get_gpl_txt)
export(ggheat)
export(hypertest)
export(interaction_to_edges)
export(intersect_all)
export(make_tcga_group)
export(match_exp_cl)
export(multi_deg)
export(multi_deg_all)
export(plcortest)
export(plot_deg)
export(point_cut)
export(quick_enrich)
export(risk_plot)
export(sam_filter)
export(surv_KM)
export(surv_cox)
export(t_choose)
export(trans_array)
export(trans_entrezexp)
export(trans_exp)
export(trans_exp_new)
export(union_all)
importFrom(BiocManager,install)
importFrom(Hmisc,rcorr)
importFrom(clusterProfiler,bitr)
importFrom(clusterProfiler,dotplot)
importFrom(clusterProfiler,enrichGO)
importFrom(clusterProfiler,enrichKEGG)
importFrom(clusterProfiler,setReadable)
importFrom(dplyr,arrange)
importFrom(dplyr,case_when)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,inner_join)
importFrom(dplyr,mutate)
importFrom(dplyr,union_all)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,stat_boxplot)
importFrom(ggplot2,stat_ellipse)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_light)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,ylim)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,makeContrasts)
importFrom(limma,topTable)
importFrom(patchwork,plot_layout)
importFrom(patchwork,plot_spacer)
importFrom(patchwork,wrap_plots)
importFrom(pheatmap,pheatmap)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_remove)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_split)
importFrom(stringr,str_starts)
importFrom(stringr,str_sub)
importFrom(stringr,str_to_lower)
importFrom(stringr,str_to_upper)
importFrom(stringr,str_wrap)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(survival,survdiff)
importFrom(survival,survfit)
importFrom(survminer,ggsurvplot)
importFrom(survminer,surv_cutpoint)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(utils,packageDescription)
