##' @importFrom utils packageDescription
.onAttach <- function(libname, pkgname) {
    packageStartupMessage("Welcome to tinyarray.
If you use tinyarray in published research, please acknowledge:
We thank Dr. Jianming Zeng (University of Macau), the members of his bioinformatics team and biotrainee, especially Xiaojie Sun, for generously sharing their experience and code.
")
}
