% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{trans_ensembl_exp}
\alias{trans_ensembl_exp}
\title{trans_ensembl_exp}
\usage{
trans_ensembl_exp(
  exp,
  mrna_only = FALSE,
  lncrna_only = FALSE,
  species = "human",
  average = FALSE
)
}
\arguments{
\item{exp}{expression matrix with Ensembl IDs as rownames}

\item{mrna_only}{only keep mrna rows in result}

\item{lncrna_only}{only keep lncrna rows in result}

\item{species}{choose human, mouse, or rat; default is human}

\item{average}{logical, if TRUE, average rows that map to the same target
symbol; if FALSE, keep the first matched row for each symbol.}
}
\value{
a transformed expression set with symbol
}
\description{
Transform rownames of an expression set from Ensembl IDs to gene symbols
using the latest gene annotation.
}
\examples{
\dontshow{if (requireNamespace("AnnotationDbi", quietly = TRUE) && requireNamespace("org.Hs.eg.db", quietly = TRUE)) withAutoprint(\{ # examplesIf}
exp = matrix(rnorm(1000),ncol = 10)
rownames(exp) = sample(mRNA_anno$gene_id,100)
colnames(exp) = c(paste0("TCGA",1:5),paste0("GTEX",1:5))
k  = trans_ensembl_exp(exp)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{trans_exp_new}}
}
\author{
Xiaojie Sun
}
