% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_geo_id.R
\name{get_gpl_txt}
\alias{get_gpl_txt}
\title{get gpl txt from GEO}
\usage{
get_gpl_txt(gpl, destdir = getwd(), download = FALSE)
}
\arguments{
\item{gpl}{gpl accession from GEO database}

\item{destdir}{The destination directory for data downloads}

\item{download}{download the txt file or not}
}
\value{
No return value. When `download = TRUE`, the GPL annotation file is
  saved in `destdir`.
}
\description{
get gpl annotation txt file from GEO database
}
\examples{
get_gpl_txt("GPL23270",destdir = tempdir())
}
\seealso{
\code{\link{geo_download}}
}
\author{
Xiaojie Sun
}
