% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{match_exp_cl}
\alias{match_exp_cl}
\title{match_exp_cl}
\usage{
match_exp_cl(exp, cl, id_column = "id", sample_centric = TRUE)
}
\arguments{
\item{exp}{TCGA  expression set}

\item{cl}{TCGA clinical data.frame}

\item{id_column}{which column contains patient IDs, either a column number or
column name}

\item{sample_centric}{logical, default TRUE; keep all samples from the same
patients. If FALSE, keep only one tumor sample for each patient.}
}
\value{
a transformed clinical data.frame with sample ids.
}
\description{
match exp and clinical data from TCGA
}
\examples{
a = match_exp_cl(exp_hub1,meta1[,2:4],"X_PATIENT")
exp_matched = a[[1]]
cl_matched = a[[2]]
b = match_exp_cl(exp_hub1,meta1[,2:4],"X_PATIENT",sample_centric = FALSE)
exp_matched = b[[1]]
cl_matched = b[[2]]
}
\seealso{
\code{\link{make_tcga_group}};\code{\link{sam_filter}}
}
\author{
Xiaojie Sun
}
