% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_geo_id.R
\name{geo_parser}
\alias{geo_parser}
\title{geo_parser}
\usage{
geo_parser(gse, destdir = getwd())
}
\arguments{
\item{gse}{GEO accession number}

\item{destdir}{The destination directory for data downloads}
}
\value{
an ExpressionSet object list
}
\description{
Legacy helper for parsing GEO series matrix files.
Prefer [geo_download()] for normal use.
}
\examples{
\dontrun{
gse = "GSE42872"
a = try(geo_parser(gse,destdir=tempdir()), silent = TRUE)
}
}
\seealso{
\code{\link{find_anno}}
}
\author{
Xiaojie Sun
}
