% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{make_tcga_group}
\alias{make_tcga_group}
\title{make_tcga_group}
\usage{
make_tcga_group(exp)
}
\arguments{
\item{exp}{sample names or an expression matrix/data.frame with sample
names in its columns}
}
\value{
a group factor with normal and tumor, corresponding to the sample
  names
}
\description{
make tcga group for given tcga sample names or expression matrix
}
\examples{
k = make_tcga_group(exp_hub1);table(k)
k2 = make_tcga_group(colnames(exp_hub1));table(k2)
}
\seealso{
\code{\link{sam_filter}};\code{\link{match_exp_cl}}
}
\author{
Xiaojie Sun
}
