% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_auto_geo_group.R
\name{print.geo_auto_group}
\alias{print.geo_auto_group}
\title{Print a GEO auto-group result}
\usage{
\method{print}{geo_auto_group}(x, ...)
}
\arguments{
\item{x}{An object returned by `auto_geo_group()`.}

\item{...}{Additional arguments passed to `print()`.}
}
\description{
Print a GEO auto-group result
}
