% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{trans_exp}
\alias{trans_exp}
\title{trans_exp}
\usage{
trans_exp(exp, mrna_only = FALSE, lncrna_only = FALSE, gtex = FALSE)
}
\arguments{
\item{exp}{expression matrix with Ensembl IDs as rownames}

\item{mrna_only}{only keep mrna rows in result}

\item{lncrna_only}{only keep lncrna rows in result}

\item{gtex}{logical, whether to include GTEx data}
}
\value{
a transformed expression set with symbol
}
\description{
Transform rownames of an expression matrix from Ensembl IDs to gene
symbols.
}
\examples{
exp = matrix(rnorm(1000),ncol = 10)
rownames(exp) = sample(mRNA_anno$gene_id,100)
colnames(exp) = paste0("sample",1:10)
k  = trans_exp(exp)
}
\seealso{
\code{\link{trans_array}}
}
\author{
Xiaojie Sun
}
