% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{sam_filter}
\alias{sam_filter}
\title{sam_filter}
\usage{
sam_filter(exp)
}
\arguments{
\item{exp}{TCGA or TCGA+GTEx expression matrix from GDC or Xena}
}
\value{
a transformed expression set without duplicated samples
}
\description{
drop duplicated samples from the same patients
}
\examples{
exp_hub1[1:4,1:4]
dim(exp_hub1)
exp2 = sam_filter(exp_hub1)
dim(exp2)
g = make_tcga_group(exp_hub1);table(g)
table(!duplicated(substr(colnames(exp_hub1[,g=="tumor"]),1,12)))
}
\seealso{
\code{\link{make_tcga_group}};\code{\link{match_exp_cl}}
}
\author{
Xiaojie Sun
}
