% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_geo_id.R
\name{find_anno}
\alias{find_anno}
\title{Find an annotation table for a GEO platform accession.}
\usage{
find_anno(gpl, install = FALSE, update = FALSE)
}
\arguments{
\item{gpl}{a GEO platform accession}

\item{install}{whether to install and library the package}

\item{update}{whether to update the package}
}
\value{
No return value. The function prints available annotation resources
  and optionally installs a Bioconductor annotation package.
}
\description{
The function prints a short explanation plus the code used to obtain
annotations. It only tests AnnoProbe `type` values when no Bioconductor
annotation package source exists for the GPL accession.
}
\examples{
\dontrun{
try(find_anno("GPL570"), silent = TRUE)
}
}
\seealso{
\code{\link{geo_download}}
}
\author{
Xiaojie Sun
}
