% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{genes}
\alias{genes}
\title{genes}
\format{
An object of class \code{character} of length 511.
}
\usage{
genes
}
\description{
Small example vector of Entrez IDs.
}
\examples{
  genes
}
\keyword{datasets}
