% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{trans_entrezexp}
\alias{trans_entrezexp}
\alias{trans_entrez_exp}
\title{trans_entrezexp}
\usage{
trans_entrezexp(entrezexp, species = "human", average = FALSE)

trans_entrez_exp(entrezexp, species = "human", average = FALSE)
}
\arguments{
\item{entrezexp}{expression matrix with Entrez IDs as rownames}

\item{species}{choose human, mouse, or rat; default is human}

\item{average}{logical, if TRUE, average rows that map to the same target
symbol; if FALSE, keep the first matched row for each symbol.}
}
\value{
a transformed expression set with symbol
}
\description{
Transform rownames of an expression set from Entrez IDs to gene symbols
using \code{clusterProfiler::bitr()}.
}
\examples{
\dontshow{if (requireNamespace("clusterProfiler", quietly = TRUE) && requireNamespace("org.Hs.eg.db", quietly = TRUE)) withAutoprint(\{ # examplesIf}
exp = matrix(rnorm(200),ncol = 10)
rownames(exp) = c("79691", "56271", "8662", "10394", "55630", "159162", "23541",
                  "79723", "54413", "22927", "92342", "23787", "5550", "8924",
                  "55274", "866", "8844", "353299", "587", "1473")
colnames(exp) = paste0("s",1:10)
exp2 = trans_entrezexp(exp)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{trans_exp}}
}
\author{
Xiaojie Sun
}
